#include "testlib.h"
#include <vector>
#include <algorithm>
#include <cmath>

#define sqr(x) ((x)*(x))

using namespace std;

const double EPS = 1e-6;
const double PI = acos(-1);

struct Point {
	long long x, y;
	
	Point() {}
	Point(long long _x, long long _y) : x(_x), y(_y) {}
	
	inline Point operator- (const Point& a) {
		return Point(x - a.x, y - a.y);
	}
	
	inline long long operator^ (const Point& p) {
		return x * p.y - y * p.x;
	}
	
	inline long long operator* (const Point& p) {
		return x * p.x + y * p.y;
	}
	
	inline long long dist2() {
		return sqr(x) + sqr(y);
	}
	
	inline double dist() {
		return sqrt(dist2());
	}
	
	inline bool operator== (const Point& p) {
		return x == p.x && y == p.y;
	}
};

int n;
vector <Point> polygon;

inline bool is_inside_triangle(Point& a, Point& b, Point& c, Point& p) {
	return ((b - a) ^ (p - a)) >= 0 && ((c - b) ^ (p - b)) >= 0 && ((a - c) ^ (p - c)) >= 0;
}

bool is_inside(Point& p) {
	if (polygon[0] == p) return true;
	if (((polygon[1] - polygon[0]) ^ (p - polygon[0])) < 0) return false;
	if (((polygon[n - 1] - polygon[0]) ^ (p - polygon[0])) > 0) return false;
	
	if (((polygon[1] - polygon[0]) ^ (p - polygon[0])) == 0)
		return is_inside_triangle(polygon[0], polygon[1], polygon[2], p);
	if (((polygon[n - 1] - polygon[0]) ^ (p - polygon[0])) == 0)
		return is_inside_triangle(polygon[0], polygon[n - 2], polygon[n - 1], p);
	
	int l = 1, r = n - 1;
	while (r - l > 1) {
		int m = (l + r) / 2;
		if (((polygon[m] - polygon[0]) ^ (p - polygon[0])) >= 0)
			l = m;
		else
			r = m;
	}
	return is_inside_triangle(polygon[0], polygon[l], polygon[l + 1], p);
}

int find_one_tangent(Point& p, int dir, int l, int r) {
	while (r - l > 1) {
		int m = (l + r) / 2;
		bool pvs = ((polygon[m] - p) ^ (polygon[(m + n - 1) % n] - p)) * dir >= 0;
		bool nxt = ((polygon[m] - p) ^ (polygon[(m + 1) % n] - p)) * dir >= 0;
		if (pvs && nxt) return m;
		if (!pvs && !nxt) {
			int p1 = find_one_tangent(p, dir, m + 1, r);
			int p2 = find_one_tangent(p, dir, l, m - 1);
			return (((polygon[p1] - p) ^ (polygon[p2] - p)) * dir > 0) ? p1 : p2;
		}
		if (!pvs) {
			if (((polygon[m] - p) ^ (polygon[l] - p)) * dir > 0) r = m - 1;
			else if (((polygon[l] - p) ^ (polygon[r] - p)) * dir > 0) r = m - 1;
			else l = m + 1;
		}
		if (!nxt) {
			if (((polygon[m] - p) ^ (polygon[l] - p)) * dir > 0) l = m + 1;
			else if (((polygon[l] - p) ^ (polygon[r] - p)) * dir > 0) r = m - 1;
			else l = m + 1;
		}
	}
	int ret = l;
	for (int i = l + 1; i <= r; i++) ret = (((polygon[ret] - p) ^ (polygon[i] - p)) * dir <= 0) ? i : ret;
	return ret;
}

pair <int, int> find_tangents(Point& p) {
	return make_pair(find_one_tangent(p, -1, 0, n - 1), find_one_tangent(p, 1, 0, n - 1));
}

inline double find_dist_segment(Point& a, Point& b, Point& p) {
	if (((b - a) * (p - a)) > 0 && ((a - b) * (p - b)) > 0)
		return 1.0 * abs((a - p) ^ (b - p)) / (a - b).dist();
	return sqrt(min((a - p).dist2(), (b - p).dist2()));
}

inline double find_dist_two_segments(Point& a, Point& b, Point& c, Point& p) {
	double d1 = find_dist_segment(a, b, p);
	double d2 = find_dist_segment(b, c, p);
	if (d1 < d2 + EPS) return d1;
	return d2;
}

double find_dist(Point& p) {
	if (is_inside(p)) return 0.0;
	
	auto pr = find_tangents(p);
	int l = pr.first, r = pr.second;
	if (r < l) r += n;
	
	while (r - l > 2) {
		int m1 = (l + l + r) / 3;
		int m2 = (l + r + r) / 3;
		if ((polygon[m1 % n] - p).dist2() <= (polygon[m2 % n] - p).dist2()) r = m2;
		if ((polygon[m1 % n] - p).dist2() >= (polygon[m2 % n] - p).dist2()) l = m1;
	}
	
	if (r - l == 1)
		return find_dist_segment(polygon[l % n], polygon[r % n], p);
	return find_dist_two_segments(polygon[l % n], polygon[(l + 1) % n], polygon[r % n], p);
}

int main(int argc, char* argv[]) {
	registerValidation(argc, argv);
	
	n = inf.readInt(3, 100'000, "n");
	inf.readEoln();
	vector <Point> p(n);
	double S = 0, P = 0;
	for (int i = 0; i < n; i++) {
		p[i].x = inf.readLong((long long)-1'000'000'000, (long long)1'000'000'000, "pxi");
		inf.readSpace();
		p[i].y = inf.readLong((long long)-1'000'000'000, (long long)1'000'000'000, "pyi");
		inf.readEoln();
		if (i) {
			S += p[i - 1] ^ p[i];
			P += (p[i - 1] - p[i]).dist();
		}
	}
	S += p[n - 1] ^ p[0];
	P += (p[n - 1] - p[0]).dist();
	S /= 2.0;
	
	polygon = p;
	
	p.push_back(p[0]);
	p.push_back(p[1]);
	for (int i = 0; i < n; i++)
		ensuref(((p[i + 1] - p[i]) ^ (p[i + 2] - p[i + 1])) > 0, "not in ccw order or not convex");
	
	vector <double> days;
	double pow2 = 1;
	for (int i = 0; i < 71; i++) {
		double a = PI, b = P, c = S * (1 - pow2);
		pow2 *= 2.0;
		days.push_back((-b + sqrt(sqr(b) - 4.0 * a * c)) / (2.0 * a));
	}
	
	int q = inf.readInt(1, 100'000, "q");
	inf.readEoln();
	for (int i = 0; i < q; i++) {
		long long x = inf.readLong((long long)-1'000'000'000, (long long)1'000'000'000, "qxi");
		inf.readSpace();
		long long y = inf.readLong((long long)-1'000'000'000, (long long)1'000'000'000, "qyi");
		inf.readEoln();
		
		Point city(x, y);
		double d = find_dist(city);
		for (int f = 1; f < 71; f++) {
		    double mk = abs(days[f] - d);
			ensuref(mk > 1e-6, "%d (%d, %d) is to close to comfort in day %d", i + 1, x, y, f);
		}
	}

	inf.readEof();
	
	return 0;
}